# 27 "syntax/xmllexer.mll"
 

type lexing_error =
    | EUnterminatedComment
    | EUnterminatedString
    | EIdentExpected
    | ECloseExpected
    | ENodeExpected
    | EAttributeNameExpected
    | EAttributeValueExpected
    | EUnterminatedEntity
    | ECamlIdentExpected
    | WNestedComments


let lex_error_to_string = function
  | EUnterminatedComment -> "Unterminated Comment"
  | EUnterminatedString  -> "Unterminated String"
  | EIdentExpected -> "Ident Expected"
  | ECloseExpected -> "Close Expected"
  | ENodeExpected -> "Node Expected"
  | EAttributeNameExpected -> "Attribute Name Expected"
  | EAttributeValueExpected -> "Attribute Value Expected"
  | EUnterminatedEntity -> "Unterminated Entity"
  | ECamlIdentExpected -> "Caml Ident Expected"
  | WNestedComments -> "Nested comments (non valid XML)"



open Camlp4
module Loc = Camlp4.PreCast.Loc
open Lexing


(* To store some context information:
     *   loc    : position of the beginning of a string, quotation and comment
     *   entity : shall we translate entities or not
*)

type context = {
  loc        : Loc.t    ;
  lexbuf     : lexbuf   ;
  buffer     : Buffer.t ;
  entity     : bool ;
}

let default_context lb = {
  loc        = Loc.ghost ;
  lexbuf     = lb        ;
  buffer     = Buffer.create 256 ;
  entity     = true ;
}

(* To buffer string literals *)

let store c = Buffer.add_string c.buffer (Lexing.lexeme c.lexbuf)
let istore_char c i = Buffer.add_char c.buffer (Lexing.lexeme_char c.lexbuf i)
let buff_contents c =
        let contents = Buffer.contents c.buffer in
        Buffer.reset c.buffer; contents

let loc c = Loc.merge c.loc (Loc.of_lexbuf c.lexbuf)

let with_curr_loc f c = f { (c) with loc = Loc.of_lexbuf c.lexbuf } c.lexbuf
let parse f c = f c c.lexbuf

let lexeme c = Lexing.lexeme c.lexbuf
let store_string c s = Buffer.add_string c.buffer s


(* Deal with entities  *)

let entities = [
  "gt"  ,62, ">";
  "lt"  ,60, "<";
  "amp" ,38, "&";
  "apos",39, "'";
  "quot",34, "\"";
]

let idents = Hashtbl.create 0
let _ = begin
  List.iter (fun (str,code,res) ->
      Hashtbl.add idents (str^";") res;
      if code > 0
      then Hashtbl.add idents ("#" ^ string_of_int code ^ ";") res
    ) entities
end


(* Update the current location with file name and line number. *)
let update_loc c line absolute chars =
        let lexbuf = c.lexbuf in
        let pos = lexbuf.lex_curr_p in
        lexbuf.lex_curr_p <- { pos with
        pos_lnum = if absolute then line else pos.pos_lnum + line;
        pos_bol = pos.pos_cnum - chars;
        }


(* The lexer is parameterized to handle both Xml and Xhtml with
   inlined Caml code modes. *)
module type LexerArgSig = sig
  (* We do not raise the same exception in both lexers *)
  val error : Camlp4.PreCast.Loc.t -> lexing_error -> exn

  type attr_name =  private [> `AttrName of string ]
  type attr_value =  private [> `AttrVal of string ]
  type attribute = private [> `Attribute of (attr_name * attr_value) ]

  type token = private [>
  | `Tag of (string * (attribute list) * bool)
  | `PCData of string
  | `CDATA of string
  | `Endtag of string
  | `Comment of string
  | `Whitespace of string
  | `Eof
  ]

  val parse_dollar_token :     context ->          lexbuf -> token
  val parse_dollar_attrname :  context -> Loc.t -> lexbuf -> attr_name
  val parse_dollar_attrvalue : context -> Loc.t -> lexbuf -> attr_value
  val parse_dollar_attribute : context -> Loc.t -> lexbuf -> attribute

end


module Make (X : LexerArgSig) = struct

let err error loc =
  raise (X.error loc error)



# 138 "syntax/xmllexer.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base =
   "\000\000\244\255\245\255\001\000\247\255\248\255\008\000\003\000\
    \010\000\255\255\012\000\017\000\252\255\000\000\000\000\002\000\
    \253\255\000\000\000\000\000\000\001\000\001\000\251\255\018\000\
    \020\000\255\255\014\000\251\255\252\255\006\000\001\000\255\255\
    \008\000\009\000\254\255\007\000\253\255\012\000\252\255\253\255\
    \011\000\255\255\254\255\002\000\254\255\000\000\014\000\255\255\
    \067\000\007\000\254\255\068\000\253\255\066\000\253\255\141\000\
    \228\000\255\255\047\001\151\000\254\255\125\001\254\255\203\001\
    \017\000\254\255\255\255\047\000\000\000\020\000\255\255\254\255\
    \002\000\009\000\009\000\000\000\253\255\034\002\253\255\254\255\
    \112\002\200\000\252\255\202\000\203\000\208\000\211\000\253\255\
    \254\255\011\000\041\000\044\000\027\000\255\255\004\002\252\255\
    \253\255\068\000\255\255\254\255\018\000\252\255\253\255\122\000\
    \255\255\254\255";
  Lexing.lex_backtrk =
   "\255\255\255\255\255\255\009\000\255\255\255\255\006\000\001\000\
    \001\000\255\255\005\000\006\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\002\000\
    \001\000\255\255\255\255\255\255\255\255\004\000\004\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \003\000\255\255\255\255\255\255\255\255\001\000\255\255\255\255\
    \003\000\255\255\255\255\000\000\255\255\255\255\255\255\002\000\
    \002\000\255\255\255\255\255\255\255\255\255\255\255\255\000\000\
    \255\255\255\255\255\255\003\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\255\255\255\255\003\000\003\000\255\255\255\255\255\255\
    \255\255\000\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\003\000\255\255\255\255\255\255\255\255\255\255\003\000\
    \255\255\255\255";
  Lexing.lex_default =
   "\003\000\000\000\000\000\003\000\000\000\000\000\255\255\255\255\
    \003\000\000\000\255\255\255\255\000\000\255\255\255\255\255\255\
    \000\000\255\255\255\255\255\255\255\255\255\255\000\000\255\255\
    \255\255\000\000\027\000\000\000\000\000\255\255\255\255\000\000\
    \255\255\255\255\000\000\255\255\000\000\038\000\000\000\000\000\
    \255\255\000\000\000\000\044\000\000\000\255\255\255\255\000\000\
    \051\000\255\255\000\000\051\000\000\000\054\000\000\000\255\255\
    \255\255\000\000\255\255\255\255\000\000\062\000\000\000\255\255\
    \065\000\000\000\000\000\255\255\255\255\255\255\000\000\000\000\
    \255\255\255\255\255\255\255\255\000\000\078\000\000\000\000\000\
    \255\255\082\000\000\000\255\255\255\255\255\255\255\255\000\000\
    \000\000\255\255\255\255\255\255\255\255\000\000\095\000\000\000\
    \000\000\255\255\000\000\000\000\101\000\000\000\000\000\255\255\
    \000\000\000\000";
  Lexing.lex_trans =
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\008\000\009\000\255\255\007\000\007\000\255\255\000\000\
    \007\000\011\000\000\000\008\000\255\255\010\000\041\000\007\000\
    \031\000\000\000\011\000\024\000\025\000\024\000\000\000\024\000\
    \008\000\024\000\032\000\007\000\004\000\255\255\005\000\255\255\
    \011\000\013\000\008\000\052\000\010\000\015\000\255\255\016\000\
    \255\255\011\000\024\000\035\000\024\000\033\000\034\000\010\000\
    \000\000\104\000\076\000\029\000\006\000\255\255\001\000\255\255\
    \010\000\019\000\021\000\017\000\018\000\036\000\255\255\012\000\
    \255\255\042\000\030\000\040\000\047\000\255\255\255\255\066\000\
    \255\255\255\255\071\000\068\000\020\000\093\000\000\000\000\000\
    \000\000\000\000\000\000\014\000\022\000\046\000\069\000\045\000\
    \000\000\000\000\000\000\000\000\000\000\055\000\099\000\049\000\
    \255\255\050\000\255\255\073\000\072\000\070\000\103\000\000\000\
    \000\000\000\000\056\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\074\000\075\000\090\000\255\255\
    \255\255\255\255\255\255\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\091\000\092\000\000\000\
    \099\000\105\000\000\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\084\000\060\000\085\000\086\000\000\000\105\000\000\000\
    \000\000\085\000\000\000\000\000\086\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \084\000\000\000\085\000\086\000\088\000\000\000\089\000\000\000\
    \085\000\087\000\088\000\086\000\089\000\000\000\000\000\087\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \002\000\255\255\255\255\000\000\000\000\083\000\000\000\000\000\
    \085\000\000\000\255\255\000\000\039\000\000\000\028\000\000\000\
    \085\000\000\000\102\000\000\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\000\000\057\000\
    \000\000\000\000\000\000\000\000\000\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\000\000\
    \000\000\000\000\000\000\255\255\255\255\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\000\000\057\000\000\000\000\000\000\000\000\000\000\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\063\000\000\000\000\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \000\000\000\000\000\000\000\000\063\000\000\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\000\000\000\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\098\000\000\000\
    \000\000\000\000\063\000\000\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\079\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\080\000\
    \000\000\000\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\000\000\000\000\000\000\
    \097\000\000\000\000\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\000\000\000\000\000\000\
    \000\000\080\000\000\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\000\000\000\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\000\000\000\000\000\000\000\000\080\000\
    \000\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\096\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000";
  Lexing.lex_check =
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\003\000\007\000\000\000\003\000\255\255\
    \007\000\006\000\255\255\008\000\008\000\010\000\037\000\008\000\
    \026\000\255\255\011\000\023\000\023\000\024\000\255\255\023\000\
    \000\000\024\000\030\000\007\000\000\000\003\000\000\000\003\000\
    \006\000\006\000\008\000\049\000\010\000\013\000\008\000\015\000\
    \008\000\011\000\023\000\029\000\024\000\032\000\033\000\006\000\
    \255\255\100\000\075\000\026\000\000\000\003\000\000\000\003\000\
    \011\000\018\000\020\000\014\000\017\000\035\000\008\000\006\000\
    \008\000\040\000\026\000\037\000\046\000\048\000\051\000\064\000\
    \048\000\051\000\069\000\067\000\019\000\092\000\255\255\255\255\
    \255\255\255\255\255\255\013\000\021\000\045\000\067\000\043\000\
    \255\255\255\255\255\255\255\255\255\255\053\000\097\000\048\000\
    \051\000\048\000\051\000\072\000\068\000\067\000\100\000\255\255\
    \255\255\255\255\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\073\000\074\000\089\000\048\000\
    \051\000\048\000\051\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\090\000\091\000\255\255\
    \097\000\103\000\255\255\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\081\000\059\000\083\000\084\000\255\255\103\000\255\255\
    \255\255\085\000\255\255\255\255\086\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \081\000\255\255\083\000\084\000\083\000\255\255\083\000\255\255\
    \085\000\083\000\085\000\086\000\085\000\255\255\255\255\085\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\003\000\043\000\255\255\255\255\081\000\255\255\255\255\
    \084\000\255\255\008\000\255\255\037\000\255\255\026\000\255\255\
    \086\000\255\255\100\000\255\255\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\255\255\056\000\
    \255\255\255\255\255\255\255\255\255\255\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\255\255\
    \255\255\255\255\255\255\048\000\051\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\255\255\058\000\255\255\255\255\255\255\255\255\255\255\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\061\000\255\255\255\255\061\000\061\000\061\000\
    \061\000\061\000\061\000\061\000\061\000\061\000\061\000\061\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\061\000\061\000\
    \061\000\061\000\061\000\061\000\061\000\061\000\061\000\061\000\
    \061\000\061\000\061\000\061\000\061\000\061\000\061\000\061\000\
    \061\000\061\000\061\000\061\000\061\000\061\000\061\000\061\000\
    \255\255\255\255\255\255\255\255\061\000\255\255\061\000\061\000\
    \061\000\061\000\061\000\061\000\061\000\061\000\061\000\061\000\
    \061\000\061\000\061\000\061\000\061\000\061\000\061\000\061\000\
    \061\000\061\000\061\000\061\000\061\000\061\000\061\000\061\000\
    \063\000\255\255\255\255\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\094\000\255\255\
    \255\255\255\255\063\000\255\255\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\077\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\077\000\
    \255\255\255\255\077\000\077\000\077\000\077\000\077\000\077\000\
    \077\000\077\000\077\000\077\000\077\000\255\255\255\255\255\255\
    \094\000\255\255\255\255\077\000\077\000\077\000\077\000\077\000\
    \077\000\077\000\077\000\077\000\077\000\077\000\077\000\077\000\
    \077\000\077\000\077\000\077\000\077\000\077\000\077\000\077\000\
    \077\000\077\000\077\000\077\000\077\000\255\255\255\255\255\255\
    \255\255\077\000\255\255\077\000\077\000\077\000\077\000\077\000\
    \077\000\077\000\077\000\077\000\077\000\077\000\077\000\077\000\
    \077\000\077\000\077\000\077\000\077\000\077\000\077\000\077\000\
    \077\000\077\000\077\000\077\000\077\000\080\000\255\255\255\255\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\255\255\255\255\255\255\255\255\080\000\
    \255\255\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\094\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255";
  Lexing.lex_base_code =
   "";
  Lexing.lex_backtrk_code =
   "";
  Lexing.lex_default_code =
   "";
  Lexing.lex_trans_code =
   "";
  Lexing.lex_check_code =
   "";
  Lexing.lex_code =
   "";
}

let rec token c lexbuf =
   __ocaml_lex_token_rec c lexbuf 0
and __ocaml_lex_token_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 176 "syntax/xmllexer.mll"
            ( update_loc c 1 false 0; `Whitespace (lexeme c) )
# 430 "syntax/xmllexer.ml"

  | 1 ->
# 177 "syntax/xmllexer.mll"
                    ( `Whitespace (lexeme c) )
# 435 "syntax/xmllexer.ml"

  | 2 ->
# 183 "syntax/xmllexer.mll"
           ( `Comment(comment c [Loc.of_lexbuf c.lexbuf] lexbuf))
# 440 "syntax/xmllexer.ml"

  | 3 ->
# 184 "syntax/xmllexer.mll"
         ( header c lexbuf;  token c lexbuf )
# 445 "syntax/xmllexer.ml"

  | 4 ->
# 185 "syntax/xmllexer.mll"
                (
     ignore (buff_contents c);
     cdata c lexbuf
     )
# 453 "syntax/xmllexer.ml"

  | 5 ->
# 189 "syntax/xmllexer.mll"
                          (
      let tag = ident_name c lexbuf in
      ignore_spaces c lexbuf;
      close_tag c lexbuf;
      `Endtag tag
    )
# 463 "syntax/xmllexer.ml"

  | 6 ->
# 195 "syntax/xmllexer.mll"
               (
      let tag = ident_name c lexbuf in
      ignore_spaces c lexbuf;
      let attribs, closed = attributes c lexbuf in
      `Tag(tag, attribs, closed)
    )
# 473 "syntax/xmllexer.ml"

  | 7 ->
# 201 "syntax/xmllexer.mll"
        (
      ignore (buff_contents c);
      store_string c (entity c lexbuf);
      `PCData (pcdata c lexbuf)
    )
# 482 "syntax/xmllexer.ml"

  | 8 ->
# 206 "syntax/xmllexer.mll"
        (
      ignore (buff_contents c);
      X.parse_dollar_token c lexbuf
    )
# 490 "syntax/xmllexer.ml"

  | 9 ->
# 210 "syntax/xmllexer.mll"
                   (
      ignore (buff_contents c);
      store c ;
      `PCData (pcdata c lexbuf)
    )
# 499 "syntax/xmllexer.ml"

  | 10 ->
# 215 "syntax/xmllexer.mll"
        ( `Eof )
# 504 "syntax/xmllexer.ml"

  | 11 ->
# 216 "syntax/xmllexer.mll"
      ( err ENodeExpected (Loc.of_lexbuf lexbuf) )
# 509 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_token_rec c lexbuf __ocaml_lex_state

and ignore_spaces c lexbuf =
   __ocaml_lex_ignore_spaces_rec c lexbuf 23
and __ocaml_lex_ignore_spaces_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 219 "syntax/xmllexer.mll"
            (
      update_loc c 1 false 0;
      ignore_spaces c lexbuf
    )
# 524 "syntax/xmllexer.ml"

  | 1 ->
# 223 "syntax/xmllexer.mll"
                      ( ignore_spaces c lexbuf )
# 529 "syntax/xmllexer.ml"

  | 2 ->
# 224 "syntax/xmllexer.mll"
       ( () )
# 534 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_ignore_spaces_rec c lexbuf __ocaml_lex_state

and comment c l lexbuf =
   __ocaml_lex_comment_rec c l lexbuf 26
and __ocaml_lex_comment_rec c l lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 234 "syntax/xmllexer.mll"
            (
      update_loc c 1 false 0;
      comment c l lexbuf
    )
# 549 "syntax/xmllexer.ml"

  | 1 ->
# 238 "syntax/xmllexer.mll"
           (
(*    warn WNestedComments (Loc.of_lexbuf lexbuf) ; *)
      store_string c "<!--" ;
      ignore (comment c (loc c :: l) lexbuf);
      store_string c "-->" ;
      comment c l lexbuf
    )
# 560 "syntax/xmllexer.ml"

  | 2 ->
# 245 "syntax/xmllexer.mll"
          (
      match l with
        | [] (* Should not occur... *)
        | [_] -> buff_contents c
        | _ -> ""
    )
# 570 "syntax/xmllexer.ml"

  | 3 ->
# 251 "syntax/xmllexer.mll"
        (
      err EUnterminatedComment
        (match l with | [] -> Loc.of_lexbuf lexbuf
                      | l :: _ -> l)
    )
# 579 "syntax/xmllexer.ml"

  | 4 ->
# 256 "syntax/xmllexer.mll"
      ( store c ; comment c l lexbuf )
# 584 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_comment_rec c l lexbuf __ocaml_lex_state

and header c lexbuf =
   __ocaml_lex_header_rec c lexbuf 37
and __ocaml_lex_header_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 260 "syntax/xmllexer.mll"
            (
      update_loc c 1 false 0;
      header c lexbuf
    )
# 599 "syntax/xmllexer.ml"

  | 1 ->
# 264 "syntax/xmllexer.mll"
         ( () )
# 604 "syntax/xmllexer.ml"

  | 2 ->
# 265 "syntax/xmllexer.mll"
        ( err ECloseExpected (loc c) )
# 609 "syntax/xmllexer.ml"

  | 3 ->
# 266 "syntax/xmllexer.mll"
      ( header c lexbuf )
# 614 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_header_rec c lexbuf __ocaml_lex_state

and cdata c lexbuf =
   __ocaml_lex_cdata_rec c lexbuf 43
and __ocaml_lex_cdata_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 269 "syntax/xmllexer.mll"
          ( `CDATA (buff_contents c) )
# 626 "syntax/xmllexer.ml"

  | 1 ->
# 270 "syntax/xmllexer.mll"
          ( store c; cdata c lexbuf )
# 631 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_cdata_rec c lexbuf __ocaml_lex_state

and pcdata c lexbuf =
   __ocaml_lex_pcdata_rec c lexbuf 48
and __ocaml_lex_pcdata_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 273 "syntax/xmllexer.mll"
            (
      store c ;
      pcdata c lexbuf
    )
# 646 "syntax/xmllexer.ml"

  | 1 ->
# 277 "syntax/xmllexer.mll"
        (
      store_string c (entity c lexbuf);
      pcdata c lexbuf
    )
# 654 "syntax/xmllexer.ml"

  | 2 ->
# 281 "syntax/xmllexer.mll"
         ( store_string c "$" ; pcdata c lexbuf )
# 659 "syntax/xmllexer.ml"

  | 3 ->
# 282 "syntax/xmllexer.mll"
       ( buff_contents c )
# 664 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_pcdata_rec c lexbuf __ocaml_lex_state

and entity c lexbuf =
   __ocaml_lex_entity_rec c lexbuf 53
and __ocaml_lex_entity_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 285 "syntax/xmllexer.mll"
                    (
      let ident = lexeme c in
      if not c.entity then "&" ^ ident else
        try Hashtbl.find idents (String.lowercase ident)
        with Not_found -> "&" ^ ident
    )
# 681 "syntax/xmllexer.ml"

  | 1 ->
# 291 "syntax/xmllexer.mll"
                       (
      let ident = lexeme c in
      if not c.entity then "&#" ^ ident else
        try Hashtbl.find idents (String.lowercase ident)
        with Not_found -> "&#" ^ ident
    )
# 691 "syntax/xmllexer.ml"

  | 2 ->
# 297 "syntax/xmllexer.mll"
            ( err EUnterminatedEntity (Loc.of_lexbuf lexbuf) )
# 696 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_entity_rec c lexbuf __ocaml_lex_state

and ident_name c lexbuf =
   __ocaml_lex_ident_name_rec c lexbuf 61
and __ocaml_lex_ident_name_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 300 "syntax/xmllexer.mll"
               ( lexeme c )
# 708 "syntax/xmllexer.ml"

  | 1 ->
# 301 "syntax/xmllexer.mll"
            ( err EIdentExpected (loc c))
# 713 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_ident_name_rec c lexbuf __ocaml_lex_state

and close_tag c lexbuf =
   __ocaml_lex_close_tag_rec c lexbuf 64
and __ocaml_lex_close_tag_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 304 "syntax/xmllexer.mll"
        ( () )
# 725 "syntax/xmllexer.ml"

  | 1 ->
# 305 "syntax/xmllexer.mll"
            ( err ECloseExpected (loc c))
# 730 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_close_tag_rec c lexbuf __ocaml_lex_state

and attributes c lexbuf =
   __ocaml_lex_attributes_rec c lexbuf 67
and __ocaml_lex_attributes_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 308 "syntax/xmllexer.mll"
         ( [], false )
# 742 "syntax/xmllexer.ml"

  | 1 ->
# 309 "syntax/xmllexer.mll"
         ( [], true )
# 747 "syntax/xmllexer.ml"

  | 2 ->
# 310 "syntax/xmllexer.mll"
             (
      let attribute = X.parse_dollar_attribute c (Loc.of_lexbuf lexbuf) lexbuf in
      ignore_spaces c lexbuf;
      let others, closed = attributes c lexbuf in
      attribute :: others, closed
    )
# 757 "syntax/xmllexer.ml"

  | 3 ->
# 316 "syntax/xmllexer.mll"
        (
      let attribute =
        let a = attr_name c lexbuf in
        let data = attr_data c lexbuf in
        `Attribute (a, data)
      in
      ignore_spaces c lexbuf;
      let others, closed = attributes c lexbuf in
      attribute :: others, closed
    )
# 771 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_attributes_rec c lexbuf __ocaml_lex_state

and attr_name c lexbuf =
   __ocaml_lex_attr_name_rec c lexbuf 77
and __ocaml_lex_attr_name_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 328 "syntax/xmllexer.mll"
               ( `AttrName (lexeme c) )
# 783 "syntax/xmllexer.ml"

  | 1 ->
# 330 "syntax/xmllexer.mll"
        (
      X.parse_dollar_attrname c (loc c) lexbuf
    )
# 790 "syntax/xmllexer.ml"

  | 2 ->
# 334 "syntax/xmllexer.mll"
            ( err EAttributeNameExpected (loc c))
# 795 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_attr_name_rec c lexbuf __ocaml_lex_state

and attr_data c lexbuf =
   __ocaml_lex_attr_data_rec c lexbuf 81
and __ocaml_lex_attr_data_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 337 "syntax/xmllexer.mll"
                                      (
      ignore(buff_contents c) ;
      X.parse_dollar_attrvalue c (loc c) lexbuf
    )
# 810 "syntax/xmllexer.ml"

  | 1 ->
# 341 "syntax/xmllexer.mll"
                          (
      ignore(buff_contents c) ;
      `AttrVal(dq_string c lexbuf)
    )
# 818 "syntax/xmllexer.ml"

  | 2 ->
# 345 "syntax/xmllexer.mll"
                           (
      ignore(buff_contents c) ;
      `AttrVal(q_string c lexbuf)
    )
# 826 "syntax/xmllexer.ml"

  | 3 ->
# 349 "syntax/xmllexer.mll"
            ( err EAttributeValueExpected (loc c))
# 831 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_attr_data_rec c lexbuf __ocaml_lex_state

and dq_string c lexbuf =
   __ocaml_lex_dq_string_rec c lexbuf 94
and __ocaml_lex_dq_string_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 352 "syntax/xmllexer.mll"
        ( buff_contents c )
# 843 "syntax/xmllexer.ml"

  | 1 ->
# 353 "syntax/xmllexer.mll"
                      (
      istore_char c 1;
      dq_string c lexbuf
    )
# 851 "syntax/xmllexer.ml"

  | 2 ->
# 357 "syntax/xmllexer.mll"
        ( err EUnterminatedString (Loc.of_lexbuf lexbuf) )
# 856 "syntax/xmllexer.ml"

  | 3 ->
# 358 "syntax/xmllexer.mll"
      (
      istore_char c 0;
      dq_string c lexbuf
    )
# 864 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_dq_string_rec c lexbuf __ocaml_lex_state

and q_string c lexbuf =
   __ocaml_lex_q_string_rec c lexbuf 100
and __ocaml_lex_q_string_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 364 "syntax/xmllexer.mll"
         ( buff_contents c )
# 876 "syntax/xmllexer.ml"

  | 1 ->
# 365 "syntax/xmllexer.mll"
                       (
      istore_char c 1;
      q_string c lexbuf
    )
# 884 "syntax/xmllexer.ml"

  | 2 ->
# 369 "syntax/xmllexer.mll"
         ( err EUnterminatedString (Loc.of_lexbuf lexbuf) )
# 889 "syntax/xmllexer.ml"

  | 3 ->
# 370 "syntax/xmllexer.mll"
      (
      istore_char c 0;
      q_string c lexbuf
    )
# 897 "syntax/xmllexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_q_string_rec c lexbuf __ocaml_lex_state

;;

# 377 "syntax/xmllexer.mll"
 
let lexing_store s buff max =
        let rec self n s =
                if n >= max then n
                else
                        match Stream.peek s with
                        | Some x ->
                                        Stream.junk s;
                                        Bytes.set buff n x;
                                        succ n
                        | _ -> n
                in
                self 0 s

let from_context c  =
        let next _ =
                let tok = with_curr_loc token c in
                let loc = Loc.of_lexbuf c.lexbuf in
                Some ((tok, loc))
                in Stream.from next

let from_lexbuf lb e =
        let c = { (default_context lb) with
        loc        = Loc.of_lexbuf lb;
                entity = e }
        in from_context c

let setup_loc lb loc =
        let start_pos = Loc.start_pos loc in
        lb.lex_abs_pos <- start_pos.pos_cnum;
        lb.lex_curr_p  <- start_pos

let from_string loc entity str =
        let lb = Lexing.from_string str in
        setup_loc lb loc;
        from_lexbuf lb entity

let from_stream loc entity strm =
        let lb = Lexing.from_function (lexing_store strm) in
        setup_loc lb loc;
        from_lexbuf lb entity

end


# 950 "syntax/xmllexer.ml"
